#include "simulator.h"
#include <string.h>
#include <cmath>

const int WINDOW_HEIGHT = 800;
const int WINDOW_WIDTH = 1200;
const int PLOT_WIDTH = 1000;
const int PLOT_HEIGHT = 500;
const int PLOT_POSX = 200;
const int PLOT_POSY = 300;
const int CALCPOINTS = 1000;

const int SIM_STEPS = 10000;
const int PLOT_POINTS = 1000;

double VALUEVECTOR[CALCPOINTS];

void Simulator::initWindow()
{
    this->videoMode = sf::VideoMode(WINDOW_WIDTH, WINDOW_HEIGHT);
    this->window = new sf::RenderWindow(this->videoMode, "Simulator" , sf::Style::Close | sf::Style::Titlebar);
    this->window->setFramerateLimit(60);

    font.loadFromFile("Fonts/Dosis-Light.ttf");
    textureSin1.loadFromFile("Images/sin1.png");
    textureSin2.loadFromFile("Images/sin2.png");
    textureTri1.loadFromFile("Images/tri1.png");
    textureTri2.loadFromFile("Images/tri2.png");
    textureSqr1.loadFromFile("Images/sq1.png");
    textureSqr2.loadFromFile("Images/sq2.png");

    backgroundU = rectanglesetup(0, 0, 300, 300, lightgray);
    backgroundList = rectanglesetup(0, 300*WINDOW_HEIGHT/800, 200, 500, lightgray);
    backgroundParameters = rectanglesetup(300*WINDOW_WIDTH/1200, 0, 700, 150, lightgray);
    backgroundPlot = rectanglesetup(PLOT_POSX, PLOT_POSY, PLOT_WIDTH, PLOT_HEIGHT, sf::Color::White);
    backgroundSetValue = rectanglesetup(300*WINDOW_WIDTH/1200, 150*WINDOW_HEIGHT/800, 700, 150, lightgray);
    backgroundUnits = rectanglesetup(1000*WINDOW_WIDTH/1200, 0, 200, 300, lightgray);
    this->rect[T] = rectanglesetup(170*WINDOW_WIDTH/1200, 115*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[A] = rectanglesetup(WINDOW_WIDTH/40, 210*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[f] = rectanglesetup(170*WINDOW_WIDTH/1200, 210*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[L] = rectanglesetup(360*WINDOW_WIDTH/1200, 40*WINDOW_HEIGHT/800, 100, 70, sf::Color::Green);
    this->rect[C] = rectanglesetup(520*WINDOW_WIDTH/1200, 40*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[R2] = rectanglesetup(680*WINDOW_WIDTH/1200, 40*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[R] = rectanglesetup(840*WINDOW_WIDTH/1200, 40*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[Unit1] = rectanglesetup(1050*WINDOW_WIDTH/1200, 20*WINDOW_HEIGHT/800, 100, 70, sf::Color::Green);
    this->rect[Unit2] = rectanglesetup(1050*WINDOW_WIDTH/1200, 115*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    this->rect[Unit3] = rectanglesetup(1050*WINDOW_WIDTH/1200, 210*WINDOW_HEIGHT/800, 100, 70, buttoncolor);
    
    this->textA.setFont(font);
    this->textA.setCharacterSize(50);
    this->textA.setFillColor(sf::Color::White);
    this->textA.setStyle(sf::Text::Bold);

    this->textf = this->textA;
    this->textA = this->textA;
    this->textT = this->textA;
    this->textL = this->textA;
    this->textC = this->textA;
    this->textR2 = this->textA;
    this->textR = this->textA;
    this->textUnit1 = this->textA;
    this->textUnit2 = this->textA;
    this->textUnit3 = this->textA;
    
    this->textA.setPosition(rect[A].getPosition().x, rect[A].getPosition().y);
    this->textf.setPosition(rect[f].getPosition().x, rect[f].getPosition().y);
    this->textT.setPosition(rect[T].getPosition().x, rect[T].getPosition().y);
    this->textL.setPosition(rect[L].getPosition().x, rect[L].getPosition().y);
    this->textC.setPosition(rect[C].getPosition().x, rect[C].getPosition().y);
    this->textR2.setPosition(rect[R2].getPosition().x, rect[R2].getPosition().y);
    this->textR.setPosition(rect[R].getPosition().x, rect[R].getPosition().y);
    this->textUnit1.setPosition(rect[Unit1].getPosition().x, rect[Unit1].getPosition().y);
    this->textUnit2.setPosition(rect[Unit2].getPosition().x, rect[Unit2].getPosition().y);
    this->textUnit3.setPosition(rect[Unit3].getPosition().x, rect[Unit3].getPosition().y);

    this->curNumText.setFillColor(sf::Color::Black);
    this->curNumText.setCharacterSize(50);
    this->curNumText.setFont(font);
    this->curNumText.setPosition(rect[L].getPosition().x*1.7, rect[L].getPosition().y*6);

    this->values.setFillColor(sf::Color::Black);
    this->values.setCharacterSize(30);
    this->values.setFont(font);
    this->values.setPosition(WINDOW_WIDTH * 0.04, WINDOW_HEIGHT * 0.4);

    this->axisUnit.setFillColor(sf::Color::Black);
    this->axisUnit.setCharacterSize(30);
    this->axisUnit.setFont(font);
    this->axisUnit.setPosition(WINDOW_WIDTH * 0.04, WINDOW_HEIGHT - 90);

    this->textA.setString("   A");
    this->textf.setString("   f");
    this->textT.setString("   T");
    this->textL.setString("   L");
    this->textC.setString("   C");
    this->textR2.setString("  R2");
    this->textR.setString("   R");
    this->textUnit1.setString("  Time");
    this->textUnit2.setString("  Bode \n   Amp");
    this->textUnit3.setString("  Bode \n   Phi");

    this->textUnit1.setCharacterSize(25);
    this->textUnit2.setCharacterSize(25);
    this->textUnit3.setCharacterSize(25);
    
    this->sin.setTexture(textureSin2);
    this->sin.setPosition(30, 20);
    this->sin.setScale(sf::Vector2f(1, 1));

    this->tri.setTexture(textureTri1);
    this->tri.setPosition(170, 20);
    this->tri.setScale(sf::Vector2f(1, 1));

    this->sqr.setTexture(textureSqr1);
    this->sqr.setPosition(30, 115);
    this->sqr.setScale(sf::Vector2f(1, 1));



    for(int i = 0; i < 9; i++)
    {
        
        this->plotline[0][i] = sf::Vertex(sf::Vector2f(PLOT_POSX, PLOT_POSY+(i+1)*50), darkgray);
        this->plotline[1][i] = sf::Vertex(sf::Vector2f(PLOT_POSX+PLOT_WIDTH, PLOT_POSY+(i+1)*50), darkgray);
        
        this->plotline[0][i+9] = sf::Vertex(sf::Vector2f(PLOT_POSX+(i+1)*100, PLOT_POSY), darkgray);
        this->plotline[1][i+9] = sf::Vertex(sf::Vector2f(PLOT_POSX+(i+1)*100, PLOT_POSY+PLOT_HEIGHT), darkgray);
        
    }
    
    this->activeUnit == 7;

}

void Simulator::buttonPressed(int x, int y) // wykrywanie wcisniecia przycisku w interfejsie
{

    for(int i = 0; i < 10; i++)
    {

            if (i <= 6) // R, L, C, t ...
            {
                if(this->rect[i].getGlobalBounds().contains(x, y))
               {
                if(i == 1 && !this->t2) this->t2 =1 ;
                else if(i == 1 && this->t2) this->t2 = 0;
                this->activeRLC = i;
               }
            }
            if (i >= 7 && i <=9) // wyj, Bode
            {
                if(this->rect[i].getGlobalBounds().contains(x, y))
               {
                this->activeUnit = i;
               }
            }

         for(int i = 0; i< 10; i++)
        {
            this->rect[i].setFillColor(this->buttoncolor);
            if( activeRLC == i || this->activeUnit == i) this->rect[i].setFillColor(sf::Color::Green);
            if(this->activeRLC == i && i == 1 && this->t2) this->rect[i].setFillColor(sf::Color::Blue);
        }
    }

    if(this->sin.getGlobalBounds().contains(x, y))
    {
        this->activeSignal = 0;
        this->sin.setTexture(textureSin2);
        this->tri.setTexture(textureTri1);
        this->sqr.setTexture(textureSqr1);
    } 
    else if(this->tri.getGlobalBounds().contains(x, y)) 
    {
        this->activeSignal = 1;
        this->sin.setTexture(textureSin1);
        this->tri.setTexture(textureTri2);
        this->sqr.setTexture(textureSqr1);
    } 
    else if(this->sqr.getGlobalBounds().contains(x, y)) 
    {
        this->activeSignal = 2;
        this->sin.setTexture(textureSin1);
        this->tri.setTexture(textureTri1);
        this->sqr.setTexture(textureSqr2);
    }
}

int Simulator::numButtonPressed()
{
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num1)) return 1;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num2)) return 2;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num3)) return 3;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num4)) return 4;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num5)) return 5;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num6)) return 6;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num7)) return 7;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num8)) return 8;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num9)) return 9;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Num0)) return 0;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Return)) return -2;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::BackSpace)) return -3;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Comma)) return -4;
    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Delete)) return -5;
    return -1;
    
}

void Simulator::inputParameter(int x) // wprowadzanie liczb
{
    if (this->aftDot < 0) this->aftDot = 0;
    if(x == -2) 
    {   
        if(this->activeRLC == 1 && this->t2) this->w = this->curNum;
        else switch (this->activeRLC)
        {
            case 0:
                this->Am = this->curNum;
                break;
            case 1:
                this->freq = this->curNum;
                break;
            case 2:
                this->Time = this->curNum;
                break;
            case 3:
                this->Lin = this->curNum;
                break;
            case 4:
                this->Cap = this->curNum;
                break;
            case 5:
                this->Res2 = this->curNum;
                break;
            case 6:
                this->Res = this->curNum;
                break;
        }
    }
    else if(x == -5) // zerowanie
    {
        this->aftDot = 0;
        this->curNum = 0;
    }
    else if(x == -4) // przecinek
    {
        this->aftDot++;
    }
    else if(x == -3) 
    {
        if(this->aftDot == 0) this->curNum = (this->curNum - static_cast<int>(this->curNum)%10)/10;
        else if(this->aftDot == 1)this->aftDot = 0;
        else
        {
            this->curNum -= (static_cast<int>(this->curNum*pow(10, aftDot-1)) % 10)/pow(10, aftDot-1);
            this->aftDot--;
        } 
    }
    else 
    {
        if(this->aftDot>0) // dodanie wcisnietej cyfry na koniec
        {
            this->curNum = this->curNum + x*pow(10, -1*this->aftDot);
            this->aftDot++;
        }
        else this->curNum = this->curNum*10 +x;
    }
}

void Simulator::sinResponse()
{
    double A = -(this->Res + this->Res2)/(this->Res * this->Res2 * this->Cap);
    double B = 1/(this->Res2 * this->Cap);
    double C = 1;
    double X = 0;
    double Y[PLOT_POINTS];
    double end = this->Time;
    int steps = SIM_STEPS;
    for (int i =0; i < steps; i++)
    {
        if(i % (SIM_STEPS/PLOT_POINTS) == 0)Y[i/(SIM_STEPS/PLOT_POINTS)] = C*X;
        X = X + end/steps * (A*X + B*this->Am*std::sin(2*3.14159*this->freq *end/steps*i));
    }
    createplot(Y, PLOT_POINTS);
}

void Simulator::triangleResponse()
{
    double A = -(this->Res + this->Res2)/(this->Res * this->Res2 * this->Cap);
    double B = 1/(this->Res2 * this->Cap);
    double C = 1;
    double X = 0;
    double Y[PLOT_POINTS];
    double end = this->Time;
    double u;
    int steps = SIM_STEPS;
    for (int i =0; i < steps; i++)
    {
        u = trianglewave(end/steps*i, 1.0/this->freq, this->Am);
        if(i % (SIM_STEPS/PLOT_POINTS) == 0)Y[i/(SIM_STEPS/PLOT_POINTS)] = C*X;
        else u = 0;

        X = X + end/steps * (A*X + B*u);
    }
    createplot(Y, PLOT_POINTS);
}

void Simulator::rectangleResponse()
{
    double A = -(this->Res + this->Res2)/(this->Res * this->Res2 * this->Cap);
    double B = 1/(this->Res2 * this->Cap);
    double C = 1;
    double X = 0;
    double Y[PLOT_POINTS];
    double end = this->Time;
    double u;
    int steps = SIM_STEPS;
    for (int i =0; i < steps; i++)
    {
        if(i % (SIM_STEPS/PLOT_POINTS) == 0)Y[i/(SIM_STEPS/PLOT_POINTS)] = C*X;
        if(end/steps*i > this->freq && end/steps*i < this->w) u = this->Am;
        else u = 0;

        X = X + end/steps * (A*X + B*u);
    }
    createplot(Y, PLOT_POINTS);
}

void Simulator::numToStr() // wyswietlanie wartosci parametrow
{
    std::stringstream ss, sss;
    ss << this->curNum;
    this->curNumText.setString(ss.str());

    sss << "R: " << this->Res <<"\n" << "R2: " << this->Res2 <<"\n" <<  "L: " << this->Lin <<"\n" << "C: " << this->Cap <<"\n" << "T: " << this->Time <<"\n" << "t2: " << this->w <<"\n" << "A: " << this->Am <<"\n" << "f / t1: " << this->freq <<"\n" ;
    this->values.setString(sss.str());
}

Simulator::Simulator()
{
    this->activeRLC = L;
    this->activeSignal = 0;
    this->activeSignalParam = A;
    this->activeUnit = Unit1;

    this->Am = 1;
    this->Res = 1000;
    this->Res2 = 1000;
    this->Cap = 500*pow(10, -6);
    this->Lin = 1;
    this->w = 1;
    this->freq = 1;
    this->Time = 1;
    this->t2 = false;

    this->dot = false;
    this->aftDot = 0;

    this->curNum = 0;

    this->initWindow();
}

Simulator::~Simulator()
{
    delete this->window;
}

bool Simulator::running()
{
    return this->window->isOpen();
}

void Simulator::pollEvents() // funkcja wykrywajaca zdarzenia i wywolujaca odpowiednie reakcje
{
    while(this->window->pollEvent(this->sfmlEvent))
    {
        if(this->sfmlEvent.type == sf::Event::Closed) this->window->close();

        if(this->sfmlEvent.type == sf::Event::MouseButtonPressed && this->sfmlEvent.mouseButton.button == sf::Mouse::Left) buttonPressed(sf::Mouse::getPosition(*window).x, sf::Mouse::getPosition(*window).y);

        if(this->sfmlEvent.type == sf::Event::KeyPressed) 
        {
            if(this->numButtonPressed() != -1) this->inputParameter(this->numButtonPressed());
        }

        if(this->activeUnit == 7) 
        {
        if(this->activeSignal==0) this->sinResponse();
        else if (this->activeSignal==1) this->triangleResponse();
        else if(this->activeSignal==2) this->rectangleResponse();
        }
        else if(this->activeUnit == 8) createplotBode(VALUEVECTOR, true);
        else if(this->activeUnit == 9) createplotBode(VALUEVECTOR, false);
    }

    
}

void Simulator::display()
{
    this->window->clear();
 
    this->window->draw(backgroundU);
    this->window->draw(backgroundList);
    this->window->draw(backgroundParameters);
    this->window->draw(backgroundSetValue);
    this->window->draw(backgroundUnits);
    this->window->draw(backgroundPlot);
    this->window->draw(sin);
    this->window->draw(tri);
    this->window->draw(sqr);
    this->window->draw(this->rect[A]);
    this->window->draw(this->rect[T]);
    this->window->draw(this->rect[f]);
    this->window->draw(this->rect[L]);
    this->window->draw(rect[C]);
    this->window->draw(rect[R2]);
    this->window->draw(rect[R]);
    this->window->draw(rect[Unit1]);
    this->window->draw(rect[Unit2]);
    this->window->draw(rect[Unit3]);
    this->window->draw(textA);
    this->window->draw(textf);
    this->window->draw(textT);
    this->window->draw(textL);
    this->window->draw(textC);
    this->window->draw(textR2);
    this->window->draw(textR);
    this->window->draw(textUnit1);
    this->window->draw(textUnit2);
    this->window->draw(textUnit3);
    this->window->draw(axisUnit);

    this->window->draw(curNumText);
    this->window->draw(values);

    sf::Vertex linedraw[2];
        for(int i = 0; i < 18; i++)
        {
            linedraw[0] = this->plotline[0][i];
            linedraw[1] = this->plotline[1][i];
            this->window->draw(linedraw, 2, sf::Lines);
        }

        for(int i = 0; i < CALCPOINTS-1; i++)
        {
            linedraw[0] = this->functionline[0][i];
            linedraw[1] = this->functionline[1][i];
            this->window->draw(linedraw, 2, sf::Lines);
        }
    for(int i = 0; i < 10; i++)
        {
            this->window->draw(valuemarker[i]);
        }
    for(int i = 0; i < 10; i++)
        {
            this->window->draw(timemarker[i]);
        }

    this->window->display();
}

sf::RectangleShape Simulator::rectanglesetup(int posx, int posy, float sizex, float sizey, sf::Color color)
{
    sf::RectangleShape rect;
    rect.setPosition(posx, posy);
    rect.setSize(sf::Vector2f(sizex, sizey));
    rect.setFillColor(color);
    rect.setOutlineColor(sf::Color::Black);
    rect.setOutlineThickness(2);
    return rect;
}

bool Simulator::hover(sf::RectangleShape rect, sf::RenderWindow& window)
{
    if(sf::Mouse::getPosition(window).x > rect.getPosition().x && sf::Mouse::getPosition(window).y > rect.getPosition().y && sf::Mouse::getPosition(window).x < rect.getPosition().x+rect.getSize().x && sf::Mouse::getPosition(window).y < rect.getPosition().x+rect.getSize().y)
    return true;
    else return false;
}


double Simulator::heaviside(double t, double hi /*moment obliczen*/) {         //jezeli druga wartosc jest wieksza od pierwszej zwroc jeden
    if(t <= hi) return 1;                                           //przeciwnie zwroc 0
    else return 0;
}

double Simulator::trianglewave(double t, double T, double A)        //t - czas probki
{                                                                   //T - okres fali
    if(t > T)                                                       //A - amplituda drgan
    {
        while(t > T)
        {
            t = t-T;
        }
    }
    double y = (4*A/T)*(heaviside(0, t)*t - 2*(t-0.25*T)*heaviside(0.25*T, t) + 2*heaviside(0.75*T, t)*(t-0.75*T) - (t-T)*heaviside(T, t));
    return y;
}

void Simulator::createplot(double A[], int calculationpoints) //calcpoints musi byc jednoczesnie wielkoscia wektora A
{
    double domain = this->Time;
    double pixelstepx = (PLOT_WIDTH/(calculationpoints-1));   //ile pixeli ma jeden krok pomiarowy
    double maxabsolutevalue = A[0];

    this->plotline[0][4] = sf::Vertex(sf::Vector2f(PLOT_POSX, PLOT_POSY+(5)*50), darkgray);
    this->plotline[1][4] = sf::Vertex(sf::Vector2f(PLOT_POSX+PLOT_WIDTH, PLOT_POSY+(5)*50), darkgray);
    this->plotline[0][13] = sf::Vertex(sf::Vector2f(PLOT_POSX+(5)*100, PLOT_POSY), darkgray);
    this->plotline[1][13] = sf::Vertex(sf::Vector2f(PLOT_POSX+(5)*100, PLOT_POSY+PLOT_HEIGHT), darkgray);

    for(int i = 0; i < calculationpoints; i++)
    {
        if(A[i] < 0 && maxabsolutevalue < -A[i]) maxabsolutevalue = -A[i];
        else if(A[i] > 0 && maxabsolutevalue < A[i]) maxabsolutevalue = A[i];
    }
    double pixelscaley = PLOT_HEIGHT/(2*maxabsolutevalue);     //ile pixeli ma jednostka na osi y
    
    for(int i = 0; i < calculationpoints-1; i++)
    {
        this->functionline[0][i] = sf::Vertex(sf::Vector2f(PLOT_POSX+i*pixelstepx, PLOT_POSY+pixelscaley*maxabsolutevalue-A[i]*pixelscaley), sf::Color::Black);
        this->functionline[1][i] = sf::Vertex(sf::Vector2f(PLOT_POSX+(i+1)*pixelstepx, PLOT_POSY+pixelscaley*maxabsolutevalue-A[i+1]*pixelscaley), sf::Color::Black);
    }

    for(int i = 0; i < 10; i++)
    {
        this->valuemarker[i].setPosition(PLOT_POSX, PLOT_POSY + PLOT_HEIGHT*i/10);
        this->valuemarker[i].setFont(font);
        this->valuemarker[i].setCharacterSize(20);
        this->valuemarker[i].setFillColor(sf::Color::Black);
        double n = maxabsolutevalue-i*maxabsolutevalue/5;
        std::string string1 = std::to_string(n);
        string1.resize(8);
        this->valuemarker[i].setString(string1);
    }
    
    std::string string1;
    for(int i = 0; i < 9; i++)
    {
        this->timemarker[i].setPosition(PLOT_POSX + (i+1)*0.1*PLOT_WIDTH, PLOT_POSY + 9*PLOT_HEIGHT/20);
        this->timemarker[i].setFont(font);
        this->timemarker[i].setCharacterSize(20);
        this->timemarker[i].setFillColor(sf::Color::Black);
        double n = (this->Time/10)*(i+1);
        string1 = std::to_string(n);
        string1.resize(7);
        this->timemarker[i].setString(string1);
    }
    this->plotline[0][4] = sf::Vertex(sf::Vector2f(PLOT_POSX, PLOT_POSY+(5)*50), sf::Color::Black);
    this->plotline[1][4] = sf::Vertex(sf::Vector2f(PLOT_POSX+PLOT_WIDTH, PLOT_POSY+(5)*50), sf::Color::Black);

    this->axisUnit.setString("Y [V]\nX [s]");
}


void Simulator::createplotBode(double A[], bool isAmplitude)
{
    double w;
    for(int i = 0; i < CALCPOINTS; i++)
    {
        w = pow(10.0, 10.0/CALCPOINTS*(i-(CALCPOINTS/2.0)));
        if(isAmplitude)
            A[i] = 20.0*log10(this->Res/sqrt(pow(this->Res + this->Res2, 2.0) + pow(w*this->Res*this->Res2*this->Cap, 2.0)));
        else 
            A[i] = (-atan2(this->Res*this->Res2*this->Cap*w,this->Res+this->Res2)/3.141592)*180;
    }
    this->plotline[0][4] = sf::Vertex(sf::Vector2f(PLOT_POSX, PLOT_POSY+(5)*50), sf::Color::Black);
    this->plotline[1][4] = sf::Vertex(sf::Vector2f(PLOT_POSX+PLOT_WIDTH, PLOT_POSY+(5)*50), sf::Color::Black);
    this->plotline[0][13] = sf::Vertex(sf::Vector2f(PLOT_POSX+(5)*100, PLOT_POSY), sf::Color::Black);
    this->plotline[1][13] = sf::Vertex(sf::Vector2f(PLOT_POSX+(5)*100, PLOT_POSY+PLOT_HEIGHT), sf::Color::Black);

    double maxabsolutevalue = A[0];
    for(int i = 0; i < CALCPOINTS; i++)
    {
        if(A[i] < 0 && maxabsolutevalue < -A[i]) maxabsolutevalue = -A[i];
        else if(A[i] > 0 && maxabsolutevalue < A[i]) maxabsolutevalue = A[i];
    }

    for(int i = 0; i < 10; i++)
    {
        this->valuemarker[i].setPosition(PLOT_POSX+PLOT_WIDTH*0.5, PLOT_POSY + PLOT_HEIGHT*i/10);
        this->valuemarker[i].setFont(font);
        this->valuemarker[i].setCharacterSize(20);
        this->valuemarker[i].setFillColor(sf::Color::Black);
        double n = maxabsolutevalue-i*maxabsolutevalue/5;
        std::string string1 = std::to_string(n);
        string1.resize(7);
        this->valuemarker[i].setString(string1);
    }

    for(int i = 0; i < 9; i++)
    {
        if(i < 5) this->timemarker[i].setPosition(PLOT_POSX + (i)*0.1*PLOT_WIDTH, PLOT_POSY + 9*PLOT_HEIGHT/20);
        else this->timemarker[i].setPosition(PLOT_POSX + (i+1)*0.1*PLOT_WIDTH, PLOT_POSY + 9*PLOT_HEIGHT/20);
        this->timemarker[i].setFont(font);
        this->timemarker[i].setCharacterSize(20);
        this->timemarker[i].setFillColor(sf::Color::Black);
        std::string string1 = "10^";
        std::string string2;
        if(i < 5) string2 = std::to_string(i-5);
        else string2 = std::to_string(i-4);
        std::string string3 = string1 + string2;
        this->timemarker[i].setString(string3);
    }

    if(isAmplitude) this->axisUnit.setString("Y [dB]\nX [w]");
    else this->axisUnit.setString("Y [degree]\nX [w]");

    double pixelstepx = PLOT_WIDTH/(CALCPOINTS-1);   //ile pixeli ma jeden krok pomiarowy
    double pixelscaley = PLOT_HEIGHT/(2*maxabsolutevalue);     //ile pixeli ma jednostka na osi y
    for(int i = 0; i < CALCPOINTS-1; i++)
    {
        this->functionline[0][i] = sf::Vertex(sf::Vector2f(PLOT_POSX+i*pixelstepx, PLOT_POSY+pixelscaley*maxabsolutevalue-A[i]*pixelscaley), sf::Color::Black);
        this->functionline[1][i] = sf::Vertex(sf::Vector2f(PLOT_POSX+(i+1)*pixelstepx, PLOT_POSY+pixelscaley*maxabsolutevalue-A[i+1]*pixelscaley), sf::Color::Black);
    }
}
